/*
* This might break with something other than glibc, new versions of glibc, i2pd and other libraries, on a different architecture, etc.
*/
#ifdef SANDBOX
#include <seccomp.h>

#include "Config.h"
#include "Log.h"
#include "Sandbox.h"

namespace i2p {
  namespace sandbox {
    int loadSeccomp() {

      int filter[] = {
        SCMP_SYS(accept),
        SCMP_SYS(accept4),
        SCMP_SYS(bind),
        SCMP_SYS(brk),
        SCMP_SYS(clock_nanosleep),
        SCMP_SYS(clone),
        SCMP_SYS(clone3),
        SCMP_SYS(close),
        SCMP_SYS(connect),
        SCMP_SYS(epoll_create),
        SCMP_SYS(epoll_create1),
        SCMP_SYS(epoll_ctl),
        SCMP_SYS(epoll_wait),
        SCMP_SYS(epoll_pwait),
        SCMP_SYS(eventfd2),
        SCMP_SYS(exit),
        SCMP_SYS(exit_group),
        SCMP_SYS(fcntl),
        SCMP_SYS(fstat),
        SCMP_SYS(ftruncate),
        SCMP_SYS(futex),
        SCMP_SYS(getdents64),
        SCMP_SYS(getpeername),
        SCMP_SYS(getpid),
        SCMP_SYS(getrandom),
        SCMP_SYS(getsockname),
        SCMP_SYS(getsockopt),
        SCMP_SYS(ioctl),
        SCMP_SYS(landlock_add_rule),
        SCMP_SYS(landlock_create_ruleset),
        SCMP_SYS(landlock_restrict_self),
        SCMP_SYS(listen),
        SCMP_SYS(lseek),
        SCMP_SYS(madvise),
        SCMP_SYS(mmap),
        SCMP_SYS(mprotect),
        SCMP_SYS(munmap),
        SCMP_SYS(newfstatat),
        SCMP_SYS(openat),
        SCMP_SYS(poll),
        SCMP_SYS(ppoll),
        SCMP_SYS(prctl),
        SCMP_SYS(prlimit64),
	#ifdef DEBUG
        SCMP_SYS(ptrace),
	#endif
        SCMP_SYS(read),
        SCMP_SYS(recvfrom),
        SCMP_SYS(recvmsg),
        SCMP_SYS(rseq),
        SCMP_SYS(rt_sigaction),
        SCMP_SYS(rt_sigprocmask),
        SCMP_SYS(rt_sigreturn),
        SCMP_SYS(sendmsg),
        SCMP_SYS(sendmmsg),
        SCMP_SYS(sendto),
        SCMP_SYS(set_robust_list),
        SCMP_SYS(setsockopt),
        SCMP_SYS(shutdown),
        SCMP_SYS(socket),
        SCMP_SYS(sysinfo),
        SCMP_SYS(timerfd_create),
        SCMP_SYS(timerfd_settime),
        SCMP_SYS(unlinkat),
        SCMP_SYS(uname),
        SCMP_SYS(write),
        SCMP_SYS(writev),
      };

      int rc;
      scmp_filter_ctx ctx;

      /* Initialize seccomp */
      ctx = seccomp_init(SCMP_ACT_KILL); // Kill the process if a violation occurs
      if (ctx == NULL) {
        LogPrint(eLogError, "Sandbox: Could not initialize seccomp");
	seccomp_release(ctx);
	return false;
      }

      /* Load rules */
      for (int i = 0; i < (int)(sizeof(filter)/sizeof(int)); i++) {
        rc = seccomp_rule_add(ctx, SCMP_ACT_ALLOW, filter[i], 0);
        if (rc != 0) {
          LogPrint(eLogError, "Sandbox: Could not add seccomp rule ", i, ": ", strerror(rc));
          seccomp_release(ctx);
	  return false;
        }
      }

      /* Load filter */
      rc = seccomp_load(ctx);
      if (rc != 0) {
        LogPrint(eLogError, "Sandbox: Could not load seccomp filter: ", strerror(rc));
	seccomp_release(ctx);
	return false;
      }

      /* Success */
      LogPrint(eLogInfo, "Sandbox: Loaded seccomp filter");
      seccomp_release(ctx);
      return true;
    }
  }
}
#endif
